/**
 * Program skeleton for the course "Programming embedded systems"
 */

#include <stdio.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "stm32f10x_conf.h"
#include "stm32f10x_gpio.h"
#include "task.h"
#include "setup.h"

/*-----------------------------------------------------------*/

#define DEBOUNCE_PERIOD (20 / portTICK_RATE_MS)

void myTask(void *params) {
  u8 state = 0;  // initially: released

  for (;;) {
    switch (state) {
    case 1: // pressed
      if (!GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0)) {
  	    vTaskDelay(DEBOUNCE_PERIOD);

        if (!GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0)) {
		  state = 0;
		  printf("released\n");
		}
	  }
	  break;
    case 0: // released
      if (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0)) {
  	    vTaskDelay(DEBOUNCE_PERIOD);

        if (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0)) {
		  state = 1;
		  printf("pressed\n");
		}
	  }
	  break;
	}

    vTaskDelay(20 / portTICK_RATE_MS);
  }
}

/*
 * Entry point of program execution
 */
int main( void )
{
  GPIO_InitTypeDef GPIO_InitStructure;

  prvSetupHardware();

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
  GPIO_Init( GPIOA, &GPIO_InitStructure );

  xTaskCreate(myTask, "GPIO reader", 100, NULL, 1, NULL);

  vTaskStartScheduler();
  assert(0);
  return 0;                 // not reachable
}
/*-----------------------------------------------------------*/


