/**
 * Program skeleton for the course "Programming embedded systems"
 */

#include <stdio.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "stm32f10x_conf.h"
#include "task.h"
#include "queue.h"
#include "setup.h"

/*-----------------------------------------------------------*/

xQueueHandle queue;

void producer(void *params) {
  int i = 0;
  for (;;) {
    printf("Producing %d\n", i);

    xQueueSend(queue,          // queue to use
	           &i,             // data to send
			   portMAX_DELAY   // max time to wait if queue is full
			   );			   // portMAX_DELAY -> wait forever

    ++i;
  }
}

void consumer(void *params) {
  int data;
  for (;;) {
    xQueueReceive(queue,          // queue to use
	              &data,          // address to write data to
			      portMAX_DELAY   // max time to wait if queue is empty
			      );

    printf("Consuming %d\n", data);

    vTaskDelay(100 / portTICK_RATE_MS);
  }
}

/*
 * Entry point of program execution
 */
int main( void )
{
  prvSetupHardware();

  queue = xQueueCreate(10,          // number of elements that can be stored
                                    // by queue
                       sizeof(int)  // size of each element in byte
					   );

  xTaskCreate(producer, "producer", 100, NULL, 1, NULL);
  xTaskCreate(consumer, "consumer", 100, NULL, 1, NULL);

  vTaskStartScheduler();
  assert(0);
  return 0;                 // not reachable
}
/*-----------------------------------------------------------*/


